use std::{io::Read, sync::{Arc, RwLock}};

use crate::util::string::minimize_whitespace;

struct WebCacheData {
    html:String,
    css:String,
    js:String,
    wasm:String,
}

#[derive(Clone)]
pub struct WebCache {
    data:Arc<RwLock<WebCacheData>>,
}
impl WebCache {
    pub fn init() -> Self
    {
        use std::fs::File;

        let mut html = String::new();
        let mut css = String::new();
        let mut js = String::new();
        let mut wasm = String::new();

        // Cache html file
        match File::open("www/.html") {
            Ok(mut file) => {
                file.read_to_string(&mut html).ok();
                html = minimize_whitespace(&html);
            }
            Err(_) => { }
        }

        // Cache css file
        match File::open("www/.css") {
            Ok(mut file) => {
                file.read_to_string(&mut css).ok();
                css = minimize_whitespace(&css);
            }
            Err(_) => { }
        }

        // Cache js file
        match File::open("www/.js") {
            Ok(mut file) => {
                file.read_to_string(&mut js).ok();
                js = minimize_whitespace(&js);
            }
            Err(_) => { }
        }

        // Cache wasm file
        match File::open("www/.wasm") {
            Ok(mut file) => {
                file.read_to_string(&mut wasm).ok();
                wasm = minimize_whitespace(&wasm);
            }
            Err(_) => { }
        }

        Self {
            data:Arc::new(RwLock::new(WebCacheData {
                html, css, js, wasm
            })),
        }
    }

    pub fn html(&self) -> String
    {
        match self.data.read() {
            Ok(reader) => reader.html.to_string(),
            Err(_) => String::new(),
        }
    }

    pub fn css(&self) -> String
    {
        match self.data.read() {
            Ok(reader) => reader.css.to_string(),
            Err(_) => String::new(),
        }
    }

    pub fn js(&self) -> String
    {
        match self.data.read() {
            Ok(reader) => reader.js.to_string(),
            Err(_) => String::new(),
        }
    }

    pub fn wasm(&self) -> String
    {
        match self.data.read() {
            Ok(reader) => reader.wasm.to_string(),
            Err(_) => String::new(),
        }
    }
}
